@@ -1,5 +1,6 @@
 import copy
 
+from ..api import APIClient
 from ..errors import (ContainerError, ImageNotFound,
                       create_unexpected_kwargs_error)
 from ..types import HostConfig
@@ -17,6 +18,24 @@ class Container(Model):
         if self.attrs.get('Name') is not None:
             return self.attrs['Name'].lstrip('/')
 
+    @property
+    def image(self):
+        """
+        The image of the container.
+        """
+        image_id = self.attrs['Image']
+        if image_id is None:
+            return None
+        return self.client.images.get(image_id.split(':')[1])
+
+    @property
+    def labels(self):
+        """
+        The labels of a container as dictionary.
+        """
+        result = self.attrs['Config'].get('Labels')
+        return result or {}
+
     @property
     def status(self):
         """
@@ -78,7 +97,7 @@ class Container(Model):
             author (str): The name of the author
             changes (str): Dockerfile instructions to apply while committing
             conf (dict): The configuration for the container. See the
-                `Remote API documentation
+                `Engine API documentation
                 <https://docs.docker.com/reference/api/docker_remote_api/>`_
                 for full details.
 
@@ -106,7 +125,7 @@ class Container(Model):
 
     def exec_run(self, cmd, stdout=True, stderr=True, stdin=False, tty=False,
                  privileged=False, user='', detach=False, stream=False,
-                 socket=False):
+                 socket=False, environment=None):
         """
         Run a command inside this container. Similar to
         ``docker exec``.
@@ -121,12 +140,14 @@ class Container(Model):
             user (str): User to execute command as. Default: root
             detach (bool): If true, detach from the exec command.
                 Default: False
-            tty (bool): Allocate a pseudo-TTY. Default: False
             stream (bool): Stream response data. Default: False
+            environment (dict or list): A dictionary or a list of strings in
+                the following format ``["PASSWORD=xxx"]`` or
+                ``{"PASSWORD": "xxx"}``.
 
         Returns:
             (generator or str): If ``stream=True``, a generator yielding
-            response chunks. A string containing response data otherwise.
+                response chunks. A string containing response data otherwise.
 
         Raises:
             :py:class:`docker.errors.APIError`
@@ -134,7 +155,7 @@ class Container(Model):
         """
         resp = self.client.api.exec_create(
             self.id, cmd, stdout=stdout, stderr=stderr, stdin=stdin, tty=tty,
-            privileged=privileged, user=user
+            privileged=privileged, user=user, environment=environment
         )
         return self.client.api.exec_start(
             resp['Id'], detach=detach, tty=tty, stream=stream, socket=socket
@@ -447,6 +468,8 @@ class ContainerCollection(Collection):
         Args:
             image (str): The image to run.
             command (str or list): The command to run in the container.
+            auto_remove (bool): enable auto-removal of the container on daemon
+                side when the container's process exits.
             blkio_weight_device: Block IO weight (relative device weight) in
                 the form of: ``[{"Path": "device_path", "Weight": weight}]``.
             blkio_weight: Block IO weight (relative weight), accepts a weight
@@ -454,12 +477,17 @@ class ContainerCollection(Collection):
             cap_add (list of str): Add kernel capabilities. For example,
                 ``["SYS_ADMIN", "MKNOD"]``.
             cap_drop (list of str): Drop kernel capabilities.
-            cpu_group (int): The length of a CPU period in microseconds.
-            cpu_period (int): Microseconds of CPU time that the container can
+            cpu_count (int): Number of usable CPUs (Windows only).
+            cpu_percent (int): Usable percentage of the available CPUs
+                (Windows only).
+            cpu_period (int): The length of a CPU period in microseconds.
+            cpu_quota (int): Microseconds of CPU time that the container can
                 get in a CPU period.
             cpu_shares (int): CPU shares (relative weight).
             cpuset_cpus (str): CPUs in which to allow execution (``0-3``,
                 ``0,1``).
+            cpuset_mems (str): Memory nodes (MEMs) in which to allow execution
+                (``0-3``, ``0,1``). Only effective on NUMA systems.
             detach (bool): Run container in the background and return a
                 :py:class:`Container` object.
             device_read_bps: Limit read rate (bytes per second) from a device
@@ -488,7 +516,12 @@ class ContainerCollection(Collection):
                 container, as a mapping of hostname to IP address.
             group_add (:py:class:`list`): List of additional group names and/or
                 IDs that the container process will run as.
+            healthcheck (dict): Specify a test to perform to check that the
+                container is healthy.
             hostname (str): Optional hostname for the container.
+            init (bool): Run an init inside the container that forwards
+                signals and reaps processes
+            init_path (str): Path to the docker-init binary
             ipc_mode (str): Set the IPC mode for the container.
             isolation (str): Isolation technology to use. Default: `None`.
             labels (dict or list): A dictionary of name-value labels (e.g.
@@ -505,21 +538,22 @@ class ContainerCollection(Collection):
                   driver.
 
             mac_address (str): MAC address to assign to the container.
-            mem_limit (float or str): Memory limit. Accepts float values
+            mem_limit (int or str): Memory limit. Accepts float values
                 (which represent the memory limit of the created container in
                 bytes) or a string with a units identification char
                 (``100000b``, ``1000k``, ``128m``, ``1g``). If a string is
                 specified without a units character, bytes are assumed as an
                 intended unit.
-            mem_limit (str or int): Maximum amount of memory container is
-                allowed to consume. (e.g. ``1G``).
             mem_swappiness (int): Tune a container's memory swappiness
                 behavior. Accepts number between 0 and 100.
             memswap_limit (str or int): Maximum amount of memory + swap a
                 container is allowed to consume.
-            networks (:py:class:`list`): A list of network names to connect
-                this container to.
             name (str): The name for this container.
+            nano_cpus (int):  CPU quota in units of 10-9 CPUs.
+            network (str): Name of the network this container will be connected
+                to at creation time. You can connect to additional networks
+                using :py:meth:`Network.connect`. Incompatible with
+                ``network_mode``.
             network_disabled (bool): Disable networking.
             network_mode (str): One of:
 
@@ -529,6 +563,7 @@ class ContainerCollection(Collection):
                 - ``container:<name|id>`` Reuse another container's network
                   stack.
                 - ``host`` Use the host network stack.
+                Incompatible with ``network``.
             oom_kill_disable (bool): Whether to disable OOM killer.
             oom_score_adj (int): An integer value containing the score given
                 to the container in order to tune OOM killer preferences.
@@ -580,10 +615,12 @@ class ContainerCollection(Collection):
             stdin_open (bool): Keep ``STDIN`` open even if not attached.
             stdout (bool): Return logs from ``STDOUT`` when ``detach=False``.
                 Default: ``True``.
-            stdout (bool): Return logs from ``STDERR`` when ``detach=False``.
+            stderr (bool): Return logs from ``STDERR`` when ``detach=False``.
                 Default: ``False``.
             stop_signal (str): The stop signal to use to stop the container
                 (e.g. ``SIGINT``).
+            storage_opt (dict): Storage driver options per container as a
+                key-value mapping.
             sysctls (dict): Kernel parameters to set in the container.
             tmpfs (dict): Temporary filesystems to mount, as a dictionary
                 mapping a path inside the container to options for that path.
@@ -624,6 +661,7 @@ class ContainerCollection(Collection):
             volumes_from (:py:class:`list`): List of container names or IDs to
                 get volumes from.
             working_dir (str): Path to the working directory.
+            runtime (str): Runtime to use with this container.
 
         Returns:
             The container logs, either ``STDOUT``, ``STDERR``, or both,
@@ -648,6 +686,12 @@ class ContainerCollection(Collection):
             raise RuntimeError("The options 'detach' and 'remove' cannot be "
                                "used together.")
 
+        if kwargs.get('network') and kwargs.get('network_mode'):
+            raise RuntimeError(
+                'The options "network" and "network_mode" can not be used '
+                'together.'
+            )
+
         try:
             container = self.create(image=image, command=command,
                                     detach=detach, **kwargs)
@@ -722,7 +766,7 @@ class ContainerCollection(Collection):
 
         Args:
             all (bool): Show all containers. Only running containers are shown
-                by default trunc (bool): Truncate output
+                by default
             since (str): Show only containers created since Id or Name, include
                 non-running ones
             before (str): Show only container created before Id or Name,
@@ -762,6 +806,10 @@ class ContainerCollection(Collection):
                                           since=since)
         return [self.get(r['Id']) for r in resp]
 
+    def prune(self, filters=None):
+        return self.client.api.prune_containers(filters=filters)
+    prune.__doc__ = APIClient.prune_containers.__doc__
+
 
 # kwargs to copy straight from run to create
 RUN_CREATE_KWARGS = [
@@ -792,10 +840,13 @@ RUN_HOST_CONFIG_KWARGS = [
     'cap_add',
     'cap_drop',
     'cgroup_parent',
+    'cpu_count',
+    'cpu_percent',
     'cpu_period',
     'cpu_quota',
     'cpu_shares',
     'cpuset_cpus',
+    'cpuset_mems',
     'device_read_bps',
     'device_read_iops',
     'device_write_bps',
@@ -806,6 +857,8 @@ RUN_HOST_CONFIG_KWARGS = [
     'dns',
     'extra_hosts',
     'group_add',
+    'init',
+    'init_path',
     'ipc_mode',
     'isolation',
     'kernel_memory',
@@ -816,6 +869,7 @@ RUN_HOST_CONFIG_KWARGS = [
     'mem_reservation',
     'mem_swappiness',
     'memswap_limit',
+    'nano_cpus',
     'network_mode',
     'oom_kill_disable',
     'oom_score_adj',
@@ -827,12 +881,14 @@ RUN_HOST_CONFIG_KWARGS = [
     'restart_policy',
     'security_opt',
     'shm_size',
+    'storage_opt',
     'sysctls',
     'tmpfs',
     'ulimits',
     'userns_mode',
     'version',
     'volumes_from',
+    'runtime'
 ]
 
 
@@ -859,10 +915,10 @@ def _create_container_args(kwargs):
     if volumes:
         host_config_kwargs['binds'] = volumes
 
-    networks = kwargs.pop('networks', [])
-    if networks:
-        create_kwargs['networking_config'] = {network: None
-                                              for network in networks}
+    network = kwargs.pop('network', None)
+    if network:
+        create_kwargs['networking_config'] = {network: None}
+        host_config_kwargs['network_mode'] = network
 
     # All kwargs should have been consumed by this point, so raise
     # error if any are left
@@ -879,5 +935,15 @@ def _create_container_args(kwargs):
                                   for p in sorted(port_bindings.keys())]
     binds = create_kwargs['host_config'].get('Binds')
     if binds:
-        create_kwargs['volumes'] = [v.split(':')[0] for v in binds]
+        create_kwargs['volumes'] = [_host_volume_from_bind(v) for v in binds]
     return create_kwargs
+
+
+def _host_volume_from_bind(bind):
+    bits = bind.split(':')
+    if len(bits) == 1:
+        return bits[0]
+    elif len(bits) == 2 and bits[1] in ('ro', 'rw'):
+        return bits[0]
+    else:
+        return bits[1]
